@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2018-2022 Greenbone Networks GmbH
+# Copyright (C) 2018-2022 Greenbone AG
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -30,6 +30,7 @@ from pathlib import Path
 from typing import Optional, Union
 
 import paramiko
+import paramiko.ssh_exception
 from lxml import etree
 
 from gvm.errors import GvmError
@@ -149,11 +150,8 @@ class GvmConnection(XmlReader):
             if self._is_end_xml():
                 break
 
-            if self._timeout is not None:
-                now = time.time()
-
-                if now > break_timeout:
-                    raise GvmError("Timeout while reading the response")
+            if time.time() > break_timeout:
+                raise GvmError("Timeout while reading the response")
 
         return response
 
@@ -193,6 +191,7 @@ class SSHConnection(GvmConnection):
         username: Optional[str] = DEFAULT_SSH_USERNAME,
         password: Optional[str] = DEFAULT_SSH_PASSWORD,
         known_hosts_file: Optional[str] = None,
+        auto_accept_host: Optional[bool] = None,
     ):
         super().__init__(timeout=timeout)
 
@@ -209,6 +208,7 @@ class SSHConnection(GvmConnection):
             if known_hosts_file is not None
             else Path.home() / DEFAULT_KNOWN_HOSTS_FILE
         )
+        self.auto_accept_host = auto_accept_host
 
     def _send_all(self, data) -> int:
         """Returns the sum of sent bytes if success"""
@@ -225,6 +225,32 @@ class SSHConnection(GvmConnection):
             data = data[sent:]
         return sent_sum
 
+    def _auto_accept_host(
+        self, hostkeys: paramiko.HostKeys, key: paramiko.PKey
+    ) -> None:
+        if self.port == DEFAULT_SSH_PORT:
+            hostkeys.add(self.hostname, key.get_name(), key)
+        elif self.port != DEFAULT_SSH_PORT:
+            hostkeys.add(
+                "[" + self.hostname + "]:" + str(self.port),
+                key.get_name(),
+                key,
+            )
+        try:
+            hostkeys.save(filename=self.known_hosts_file)
+        except OSError as e:
+            raise GvmError(
+                "Something went wrong with writing "
+                f"the known_hosts file: {e}"
+            ) from None
+        key_type = key.get_name().replace("ssh-", "").upper()
+        logger.info(
+            "Warning: Permanently added '%s' (%s) to "
+            "the list of known hosts.",
+            self.hostname,
+            key_type,
+        )
+
     def _ssh_authentication_input_loop(
         self, hostkeys: paramiko.HostKeys, key: paramiko.PKey
     ) -> None:
@@ -243,7 +269,14 @@ class SSHConnection(GvmConnection):
         add = input()
         while True:
             if add == "yes":
-                hostkeys.add(self.hostname, key.get_name(), key)
+                if self.port == DEFAULT_SSH_PORT:
+                    hostkeys.add(self.hostname, key.get_name(), key)
+                elif self.port != DEFAULT_SSH_PORT:
+                    hostkeys.add(
+                        "[" + self.hostname + "]:" + str(self.port),
+                        key.get_name(),
+                        key,
+                    )
                 # ask user if the key should be added permanently
                 print(
                     f"Do you want to add {self.hostname} "
@@ -291,6 +324,7 @@ class SSHConnection(GvmConnection):
         """Get the remote host key for ssh connection"""
         try:
             tmp_socket = socketlib.socket()
+            tmp_socket.settimeout(self._timeout)
             tmp_socket.connect((self.hostname, self.port))
         except OSError as e:
             raise GvmError(
@@ -331,12 +365,20 @@ class SSHConnection(GvmConnection):
                     f"the known_hosts file: {e}"
                 ) from None
         hostkeys = self._socket.get_host_keys()
-        if not hostkeys.lookup(self.hostname):
+        # Switch based on SSH Port
+        if self.port == DEFAULT_SSH_PORT:
+            hostname = self.hostname
+        else:
+            hostname = f"[{self.hostname}]:{self.port}"
+
+        if not hostkeys.lookup(hostname):
             # Key not found, so connect to remote and fetch the key
             # with the paramiko Transport protocol
             key = self._get_remote_host_key()
-
-            self._ssh_authentication_input_loop(hostkeys=hostkeys, key=key)
+            if self.auto_accept_host:
+                self._auto_accept_host(hostkeys=hostkeys, key=key)
+            else:
+                self._ssh_authentication_input_loop(hostkeys=hostkeys, key=key)
 
     def connect(self) -> None:
         """
@@ -364,6 +406,7 @@ class SSHConnection(GvmConnection):
             paramiko.BadHostKeyException,
             paramiko.AuthenticationException,
             paramiko.SSHException,
+            paramiko.ssh_exception.NoValidConnectionsError,
             ConnectionError,
         ) as e:
             raise GvmError(f"SSH Connection failed: {e}") from None
@@ -503,8 +546,10 @@ class UnixSocketConnection(GvmConnection):
             self._socket.connect(self.path)
         except FileNotFoundError:
             raise GvmError(f"Socket {self.path} does not exist") from None
-        except ConnectionError:
-            raise GvmError(f"Could not connect to socket {self.path}") from None
+        except ConnectionError as e:
+            raise GvmError(
+                f"Could not connect to socket {self.path}. Error was {e}"
+            ) from None
 
 
 class DebugConnection:
